<?php

/**
 * @file
 * Builds placeholder replacement tokens for aws-related data.
 */

use Drupal\Core\Render\BubbleableMetadata;

/**
 * Implements hook_token_info().
 */
function aws_cloud_token_info() {
  $types = [
    'name' => t('AWS Cloud Instances'),
    'description' => t('Tokens related to individual aws cloud instances.'),
    'needs-data' => 'aws_cloud_instance',
  ];
  $instance['name'] = [
    'name' => t('Instance Name'),
    'description' => t('The name of the Instance entity'),
  ];
  $instance['id'] = [
    'name' => t('Instance ID'),
    'description' => t('AWS Instance ID'),
  ];
  $instance['launch_time'] = [
    'name' => t('Launch Time'),
    'description' => t('The time the instance launched'),
  ];
  $instance['instance_state'] = [
    'name' => t('Instance State'),
    'description' => t('The current state of the instance'),
  ];
  $instance['availability_zone'] = [
    'name' => t('Availability Zone'),
    'description' => t('Zone the instance is in'),
  ];
  $instance['private_ip'] = [
    'name' => t('Private IP Address'),
    'description' => t('The instance private ip'),
  ];
  $instance['public_ip'] = [
    'name' => t('Public IP Address'),
    'description' => t('The instance public ip'),
  ];
  $instance['elastic_ip'] = [
    'name' => t('Elastic IP Address'),
    'description' => t('The instance elastic ip address'),
  ];
  $instance['instance_link'] = [
    'name' => t('Instance Link'),
    'description' => t('Absolute link to the instance'),
  ];
  $instance['instance_link_edit'] = [
    'name' => t('Edit Instance Link'),
    'description' => t('Absolute link to edit the instance'),
  ];
  // Extra tokens can be added here.
  return [
    'types' => $types,
    'tokens' => ['aws_cloud_instance' => $instance],
  ];
}

/**
 * Implements hook_tokens().
 */
function aws_cloud_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {

  $replacements = [];
  if ($type == 'aws_cloud_instance' && !empty($data['aws_cloud_instance'])) {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Instance $instance */
    $instance = $data['aws_cloud_instance'];
    foreach ($tokens as $name => $original) {
      switch ($name) {
        case 'name':
          $replacements[$original] = $instance->getName();
          break;

        case 'id':
          $replacements[$original] = $instance->getInstanceId();
          break;

        case 'launch_time':
          $replacements[$original] = $instance->getLaunchTime();
          break;

        case 'instance_state':
          $replacements[$original] = $instance->getInstanceState();
          break;

        case 'availability_zone':
          $replacements[$original] = $instance->getAvailabilityZone();
          break;

        case 'private_ip':
          $replacements[$original] = $instance->getPrivateIps();
          break;

        case 'public_ip':
          $replacements[$original] = $instance->getPublicIp();
          break;

        case 'elastic_ip':
          $replacements[$original] = $instance->getElasticIp();
          break;

        case 'instance_link':
          $replacements[$original] = $instance->toUrl('canonical', ['absolute' => TRUE])->toString();
          break;

        case 'instance_link_edit':
          $replacements[$original] = $instance->toUrl('edit-form', ['absolute' => TRUE])->toString();
          break;
      }
    }
  }
  return $replacements;
}
